@@ -1,7 +1,7 @@
 #!/usr/bin/python
 
-from __future__ import unicode_literals
 
+import io as StringIO
 import math
 import re
 
@@ -9,23 +9,16 @@ from ..metrics_core import Metric, METRIC_LABEL_NAME_RE
 from ..samples import Exemplar, Sample, Timestamp
 from ..utils import floatToGoString
 
-try:
-    import StringIO
-except ImportError:
-    # Python 3
-    import io as StringIO
-
 
 def text_string_to_metric_families(text):
     """Parse Openmetrics text format from a unicode string.
 
     See text_fd_to_metric_families.
     """
-    for metric_family in text_fd_to_metric_families(StringIO.StringIO(text)):
-        yield metric_family
+    yield from text_fd_to_metric_families(StringIO.StringIO(text))
 
 
-_CANONICAL_NUMBERS = set([float("inf")])
+_CANONICAL_NUMBERS = {float("inf")}
 
 
 def _isUncanonicalNumber(s):
@@ -83,7 +76,7 @@ def _unescape_help(text):
 def _parse_value(value):
     value = ''.join(value)
     if value != value.strip() or '_' in value:
-        raise ValueError("Invalid value: {0!r}".format(value))
+        raise ValueError(f"Invalid value: {value!r}")
     try:
         return int(value)
     except ValueError:
@@ -95,7 +88,7 @@ def _parse_timestamp(timestamp):
     if not timestamp:
         return None
     if timestamp != timestamp.strip() or '_' in timestamp:
-        raise ValueError("Invalid timestamp: {0!r}".format(timestamp))
+        raise ValueError(f"Invalid timestamp: {timestamp!r}")
     try:
         # Simple int.
         return Timestamp(int(timestamp), 0)
@@ -108,7 +101,7 @@ def _parse_timestamp(timestamp):
             # Float.
             ts = float(timestamp)
             if math.isnan(ts) or math.isinf(ts):
-                raise ValueError("Invalid timestamp: {0!r}".format(timestamp))
+                raise ValueError(f"Invalid timestamp: {timestamp!r}")
             return ts
 
 
@@ -359,7 +352,7 @@ def _parse_remaining_text(text):
     ts = _parse_timestamp(timestamp)
     exemplar = None
     if exemplar_labels is not None:
-        exemplar_length = sum([len(k) + len(v) for k, v in exemplar_labels.items()])
+        exemplar_length = sum(len(k) + len(v) for k, v in exemplar_labels.items())
         if exemplar_length > 128:
             raise ValueError("Exmplar labels are too long: " + text)
         exemplar = Exemplar(
@@ -562,10 +555,16 @@ def text_fd_to_metric_families(fd):
 
             if typ == 'stateset' and name not in sample.labels:
                 raise ValueError("Stateset missing label: " + line)
-            if (typ in ['histogram', 'gaugehistogram'] and name + '_bucket' == sample.name
+            if (name + '_bucket' == sample.name
                     and (sample.labels.get('le', "NaN") == "NaN"
                          or _isUncanonicalNumber(sample.labels['le']))):
                 raise ValueError("Invalid le label: " + line)
+            if (name + '_bucket' == sample.name
+                    and (not isinstance(sample.value, int) and not sample.value.is_integer())):
+                raise ValueError("Bucket value must be an integer: " + line)
+            if ((name + '_count' == sample.name or name + '_gcount' == sample.name)
+                    and (not isinstance(sample.value, int) and not sample.value.is_integer())):
+                raise ValueError("Count value must be an integer: " + line)
             if (typ == 'summary' and name == sample.name
                     and (not (0 <= float(sample.labels.get('quantile', -1)) <= 1)
                          or _isUncanonicalNumber(sample.labels['quantile']))):
